import unittest
from unittest.mock import patch
import  src.log_utility as log_utility

# init the test class
class TestLogUtility(unittest.TestCase):
    """_summary_

    Args:
        unittest (_type_): _description_
    """
    # test function preprocessing_output_directory in log_utility.py
    def test_preprocessing_output_directory(self):
        # assume the output directory is /tmp/log_trans/
        # there are 12 different files in the directory
        # file name like : 20210913175500-log.tar.gz
        # mock files in the directory and its create time is sort by name
        mock_files = [
            "20210913175500-log.tar.gz",
            "20210913175501-log.tar.gz",
            "20210913175502-log.tar.gz",
            "20210913175503-log.tar.gz",
            "20210913175504-log.tar.gz",
            "20210913175505-log.tar.gz",
            "20210913175506-log.tar.gz",
            "20210913175507-log.tar.gz",
            "20210913175508-log.tar.gz",
            "20210913175509-log.tar.gz",
            "20210913175510-log.tar.gz",
            "20210913175511-log.tar.gz",
        ]
        # mock the os.listdir function
        with patch("os.listdir") as mock_listdir:
            # mock the return value of os.listdir
            mock_listdir.return_value = mock_files
            # mock the os.remove function
            with patch("os.remove") as mock_remove:
                # call the preprocessing_output_directory function
                log_utility.preprocessing_output_directory("/tmp/log_trans/")
                # assert the mock_remove function is called 2 times
                mock_remove.assert_has_calls(mock_remove, 2)

                # make sure remove the right file
                mock_remove.assert_has_calls(mock_remove, "20210913175500-log.tar.gz")
                mock_remove.assert_has_calls(mock_remove, "20210913175501-log.tar.gz")



if __name__ == "__main__":
    unittest.main()



import unittest
import os
from unittest.mock import patch
import json
from src.v_t_chart import *
from datetime import datetime


class TestReadSortData(unittest.TestCase):

    def test_load_json_data_all_zip_files(self):
        with patch("src.v_t_chart.datetime") as mock_date_time:
            mock_date_time.now.return_value = datetime(2022, 8, 23, 14, 13, 27)
            mock_date_time.side_effect = lambda *args, **kw: datetime(*args, **kw)

            data_files = set(get_file_in_time_range_sort("G27A25S", "./test/test_data/sort", 1, mock_date_time.now()))

            self.assertIsInstance(data_files, set)

            self.assertIn(os.path.abspath("./test/test_data/sort/shuttle-debug-G27A25S/shuttle-debug-G27A25S-2022-08-23.log.gz"), data_files)
            self.assertEqual(len(data_files), 1)

            # diagram_input_data = collection_data(data_files, parse("2022-08-29 14:45:20"), 1)


    def test_read_data_lines_called_once(self):
        with patch("src.v_t_chart.datetime") as mock_date_time:
            mock_date_time.now.return_value = datetime(2022, 8, 23, 14, 16, 27)
            mock_date_time.side_effect = lambda *args, **kw: datetime(*args, **kw)

            with patch("src.v_t_chart.read_data_lines") as mock_read_data_lines:

                data_files = set(get_file_in_time_range_sort("G27A25S", "./test/test_data/sort", 1, mock_date_time.now()))

                self.assertIsInstance(data_files, set)

                self.assertIn(os.path.abspath("./test/test_data/sort/shuttle-debug-G27A25S/shuttle-debug-G27A25S-2022-08-23.log.gz"), data_files)

                self.assertEqual(len(data_files), 2)

                # collection_data(data_files, parse("2022-08-29 14:45:20"), 1)
                collection_data_for_plot(data_files, parse("2022-08-29 14:45:20"), 1)

                # self.assertEqual(mock_read_data_lines(),None)
                self.assertEqual(mock_read_data_lines.call_count, 2)


    def test_load_right_data(self):
        with patch("src.v_t_chart.datetime") as mock_date_time:
            mock_date_time.now.return_value = datetime(2022, 8, 24, 15, 56, 00)
            mock_date_time.side_effect = lambda *args, **kw: datetime(*args, **kw)

            data_files = set(get_file_in_time_range_sort("G27A25S", "./test/test_data/sort", 1, mock_date_time.now()))

            self.assertIsInstance(data_files, set)

            self.assertEqual(len(data_files), 2)
            input_data = collection_data_for_plot(data_files, mock_date_time.now(), 1)
            print(input_data)



if __name__ == "main":
    unittest.main()



import unittest
from unittest.mock import patch
from src.v_t_chart import *
from datetime import datetime


class TestTimeRange(unittest.TestCase):

    def test_is_file_in_time_range(self):
        with patch("src.v_t_chart.datetime") as mock_date_time:
            mock_date_time.now.return_value = datetime(2022, 8, 23, 14, 16, 27)
            mock_date_time.side_effect = lambda *args, **kw: datetime(*args, **kw)

            res = is_file_in_time_range("./test/test_data/sort/shuttle-debug-G27A25S/shuttle-debug-G27A25S-2022-08-23.log.gz", 1, mock_date_time.now())
            self.assertEqual(res, True)

    def test_load_json_data_all_zip_files(self):
        with patch("src.v_t_chart.datetime") as mock_date_time:
            mock_date_time.now.return_value = datetime(2022, 8, 23, 14, 16, 27)
            mock_date_time.side_effect = lambda *args, **kw: datetime(*args, **kw)

            res = set(get_file_in_time_range_sort("G27A25S", "./test/test_data/sort", 10, mock_date_time.now()))
            self.assertIsInstance(res, set)
            self.assertIn(os.path.abspath("./test/test_data/sort/shuttle-debug-G27A25S/shuttle-debug-G27A25S-2022-08-23.log.gz"), res)
            self.assertIn(os.path.abspath("./test/test_data/sort/shuttle-debug-G27A25S/shuttle-debug-G27A25S-2022-08-23.log.1.gz"), res)
            self.assertNotIn(os.path.abspath("test/test_data/sort/shuttle-debug-G27A25S/shuttle-debug-G27A25S-2022-08-23.log.2"), res)

    def test_load_json_data_general_files(self):
        with patch("src.v_t_chart.datetime") as mock_date_time:
            mock_date_time.now.return_value = datetime(2022, 8, 23, 15, 45, 47)
            mock_date_time.side_effect = lambda *args, **kw: datetime(*args, **kw)

            res = set(get_file_in_time_range_sort("G27A25S", "./test/test_data/sort", 10, mock_date_time.now()))
            self.assertIsInstance(res, set)
            self.assertEqual(len(res), 2)
            self.assertNotIn("./test/test_data/sort/shuttle-debug-G27A25S/shuttle-debug-G27A25S-2022-08-23.log.gz", res)

    def test_load_json_data_one_file(self):
        with patch("src.v_t_chart.datetime") as mock_date_time:
            mock_date_time.now.return_value = datetime(2022, 8, 23, 17, 45, 47)
            mock_date_time.side_effect = lambda *args, **kw: datetime(*args, **kw)

            # 测试取前后60分钟的数据
            res = set(get_file_in_time_range_sort("G27A25S", "./test/test_data/sort", 60, mock_date_time.now()))
            self.assertIsInstance(res, set)
            self.assertEqual(len(res), 1)
            self.assertIn(os.path.abspath("./test/test_data/sort/shuttle-debug-G27A25S/shuttle-debug-G27A25S-2022-08-23.log.2"), res)

    def test_should_get_no_data(self):
        with patch("src.v_t_chart.datetime") as mock_date_time:
            mock_date_time.now.return_value = datetime(2022, 8, 24, 15, 25, 50)
            # 2022-08-24T15:35:50 ~ 2022-08-24T15:55:50
            res = set(get_file_in_time_range_sort("G27A25S", "./test/test_data/sort", 5, mock_date_time.now()))

            input_data = collection_data_for_plot(res, mock_date_time.now(), 1)

            self.assertEqual(input_data, ([], [], []))

    def test_collection_data(self):
        with patch("src.v_t_chart.datetime") as mock_date_time:
            mock_date_time.now.return_value = datetime(2022, 8, 24, 15, 25, 50)
            # 2022-08-24T15:35:50 ~ 2022-08-24T15:55:50
            res = set(get_file_in_time_range_sort("G27A25S", "./test/test_data/sort", 30, mock_date_time.now()))

            input_data = collection_data_for_plot(res, mock_date_time.now(), 30)

            self.assertNotEqual(input_data[0], [])


# run: python3 -m test.test_can_io_config.sort_excel_test
if __name__ == '__main__':
    unittest.main()



# -*- coding: utf-8 -*-

import unittest
from unittest.mock import patch
from datetime import datetime
import re
from src.log_utility import *


class TestLogFileNameMatch(unittest.TestCase):

    def test_pick_load_file_match(self):
        res1 = pick_load_file_match_res("load-2022-08-18.log")
        self.assertEqual(res1.group("log_serial_number"), None)

        res2 = pick_load_file_match_res("load-2022-08-18.log.1")

        res3 = pick_load_file_match_res("load-2022-08-18.log.gz")
        self.assertEqual(res3.group("log_zip_suffix"), ".gz")
        self.assertEqual(res3.group("log_serial_number"), None)

        res4 = pick_load_file_match_res("load-2022-08-18.log.1.gz")
        self.assertEqual(bool(res1), True)
        self.assertEqual(bool(res2), True)
        self.assertEqual(bool(res3), True)
        self.assertEqual(bool(res4), True)
        self.assertEqual(res4.group("log_zip_suffix"), ".gz")
        self.assertEqual(res4.group("log_serial_number"), ".1")

# run: python3 -m test.test_can_io_config.sort_excel_test
if __name__ == '__main__':
    unittest.main()



import src.pack_log
import src.v_t_chart
import argparse
import fcntl
import sys

def entry():
    # only can run one process at a time
    lock_file = open("/tmp/log-tool-lock", "w")
    try:
        fcntl.flock(lock_file, fcntl.LOCK_EX | fcntl.LOCK_NB)
    except IOError:
        print("another process is running, exit")
        sys.exit(-1)


    parser = argparse.ArgumentParser(formatter_class=argparse.ArgumentDefaultsHelpFormatter)
    parser.set_defaults(func=lambda args: parser.print_help())

    subparsers = parser.add_subparsers(title='subcommands', dest='command', help='sub-command help')

    subparsers_pack = subparsers.add_parser("pack", help='pack help')
    src.pack_log.init_input_args(subparsers_pack)
    subparsers_pack.set_defaults(func=src.pack_log.pack_log)

    subparsers_chart = subparsers.add_parser("chart", help='chart help')
    src.v_t_chart.init_input_args(subparsers_chart)
    subparsers_chart.set_defaults(func=src.v_t_chart.generate_chart)

    # parse the args and call whatever function was selected
    args = parser.parse_args()
    args.func(args)


if __name__=="__main__":
    sys.exit(entry())



from datetime import datetime, timedelta
from sqlite3 import Timestamp
from dateutil.parser import parse
import gzip
import json
import os
from os import SEEK_END
import matplotlib.pyplot as plt
import matplotlib.dates as mdates
import matplotlib.ticker as ticker
import re



def is_file_in_time_range(file_path, delta_mins=10, date_selected=datetime.now()):
    """检测文件是否符合期望的时间要求

    Args:
        file_path (_type_): 文件路径
        delta_mins (int, optional): 期望的时间窗口，前后N分钟
        date_selected (_type_, optional): 日志查询窗口的中心时间戳

    Returns:
        _type_: _description_
    """
    try:
        mt = os.path.getmtime(file_path)
        ct = os.path.getctime(file_path)

        time_range_start_timestamp = (date_selected - timedelta(minutes=delta_mins)).timestamp()
        time_range_end_timestamp = (date_selected + timedelta(minutes=delta_mins)).timestamp()

        # 最后编辑时间小于范围开始时间
        if mt < time_range_start_timestamp:
            print(f"{file_path} not in time range")
            return False

        if file_path.endswith("gz"):
            print(f"{file_path} is checking ...")
            with gzip.open(os.path.join(file_path), mode="rt") as f:
                return check_line_in_a_file(f, delta_mins, date_selected)
        else:
            print(f"{file_path} is checking ...")
            with open(os.path.join(file_path), mode="r", encoding="utf-8") as f:
                return check_line_in_a_file(f, delta_mins, date_selected)

    except FileNotFoundError as file_not_find_error:
        print(str(file_not_find_error))


def check_line_in_a_file(f, delta_mins, date_selected=datetime.now()):
    """检测数据是否符合时间预期

    Args:
        f (_type_): 已经打开的文件
        delta_mins (int, optional): 期望的时间窗口，前后N分钟
        date_selected (_type_, optional): 日志查询窗口的中心时间戳

    Returns:
        _type_: _description_
    """
    if f.closed:
        return False

    try:
        first_line = f.readline()
        # read list line most efficient from f
        f.seek(0, SEEK_END)
        size = f.tell()
        f.seek(max(size-1024, 0), 0)
        lines = f.readlines()
        last_line = lines[-1]


        if not first_line and not last_line:
            return False

        log_start_timestamp_string = json.loads(first_line)["timestamp"]
        log_end_timestamp_string = json.loads(last_line)["timestamp"]
    except Exception:
        try:
            #2022-09-14T19:00:02.778000+
            log_start_timestamp_string = find_datetime_in_line(first_line)
            log_end_timestamp_string = find_datetime_in_line(last_line)
        except Exception:
            return False

    log_start_timestamp = parse(log_start_timestamp_string).timestamp()
    log_end_timestamp = parse(log_end_timestamp_string).timestamp()

    # print(f"{log_start_timestamp} --- {log_start_timestamp}")

    time_range_start_timestamp = (date_selected - timedelta(minutes=delta_mins)).timestamp()
    time_range_end_timestamp = (date_selected + timedelta(minutes=delta_mins)).timestamp()

    if time_range_start_timestamp > log_end_timestamp:
        return False

    if time_range_end_timestamp < log_start_timestamp:
        return False

    return True


def find_datetime_in_line(line):
    '''从一行文本中找到时间字符串'''
    # datetime like: 2022-11-07T08:48:24.269000+0800
    # if match return it, else return None
    try:
        return re.findall(r"\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{6}\+\d{4}", line)[0]
    except Exception:
        return None


def find_voltage_in_line(line):
    '''从一行文本中找到电压值'''
    # voltage like "v 26.95" ofr "v 27"
    try:
        return float(re.findall(r"v\s(\d+\.\d+$|\d+$)", line)[0])
    except Exception:
        return None


def find_torque_in_line(line):
    '''从一行文本中找到扭矩值'''
    try:
        return int(re.findall(r"t\s(-?\d+)", line)[0])
    except Exception:
        return None


def read_data_lines(data_f, date_selected, time_range):
    """ 读取log数据，并进行清洗

    Args:
        f (file_stream): 打开的文件描述符
    """
    if data_f.closed:
        raise RuntimeError("文件描述符不存在")

    # print(f"time_range: {time_range}")

    time_range_start_timestamp = (date_selected - timedelta(minutes=time_range)).timestamp()
    time_range_end_timestamp = (date_selected + timedelta(minutes=time_range)).timestamp()

    timestamp_points = []
    cur_voltage_values = []
    torque_values = []

    json_line = True

    for line in data_f:
        # check if line is json format

        try:
            data = json.loads(line.strip("\n"))
            json_line = True
        except Exception:
            json_line = False
            if "n - c" in line:
                continue
            # data line maybe like this: 2022-11-07T08:48:23.011000+0800 TRACE:         r - c -69212477 cv -5485 t -1399 T LT3 o 18.71 rf {"10.0.64.121":20,"10.0.64.122":100,"total":58} v 23.63
        # parse it
            data = {}
            data["timestamp"] = find_datetime_in_line(line)
            data["v"] = find_voltage_in_line(line)
            data["t"] = find_torque_in_line(line)

        # 选取传入时间戳范围内的所有时间
        log_line_time = parse(data["timestamp"])
        log_line_timestamp = log_line_time.timestamp()

        if log_line_timestamp < time_range_start_timestamp or log_line_timestamp > time_range_end_timestamp:
            continue

        # 获取相关的电压，扭矩，时间数据
        if (('message' in data.keys()) == False):
            if json_line and data["type"] == "n":
                continue

            if not data['v'] or not data["t"]:
                continue

            timestamp_points.append(log_line_time)
            cur_voltage_values.append(data['v'])
            torque_values.append(data['t'])
        else:
            timestamp_points.append(log_line_time)
            cur_voltage_values.append(json.loads(data['message'])["curVoltageMainPower"])
            torque_values.append(json.loads(data['message'])["torque_actual_value"])

    return (timestamp_points, cur_voltage_values, torque_values)


def get_file_in_time_range(input_args, **kwargs):
    """_summary_

    Args:
        input_args (_type_): _description_

    Raises:
        RuntimeError:  输入错误的参数，kwargs['download_log']不为True的情况下，input_args的值不为"sort"或"pick"

    Returns:
        _type_: _description_
    """

    # if input_args.log_root_dir is None or empty, raise error
    if not input_args.log_root_dir:
        raise FileNotFoundError(f"--log_root_dir 不能为空")

    if "download_log" in kwargs and kwargs["download_log"]:
        return get_file_in_time_range_for_download(input_args.log_root_dir, input_args.time_range, parse(input_args.timestamp), **kwargs)

    # if input_args.robot_id is empty or None, raise RuntimeError
    if not input_args.robot_id:
        raise RuntimeError("robot_id is empty")

    if input_args.prd_type == "sort":
        return get_file_in_time_range_sort(input_args.robot_id, input_args.log_root_dir, input_args.time_range, parse(input_args.timestamp))

    if input_args.prd_type == "pick":
        return get_file_in_time_range_pick(input_args.robot_id, input_args.log_root_dir, input_args.time_range, parse(input_args.timestamp))

    raise RuntimeError("prd_type 参数错误， 不是sort或pick")


def get_file_in_time_range_sort(robot_id,  log_root_folder, time_range_in_mins=10, date_selected=datetime.now()):
    """ 遍历所有文件并保存所有日志文件

    Args:
        robot_id (string): 机器人编号
        log_root_folder (string): 日志文件根目录
        time_range_in_mins (int, optional): 基于选取日志的时间，确定选取日志范围的时间窗口
        date_selected (datetime, optional): 选取日志的时间

    Yields:
        generator(string): 目标文件路径的生成器
    """
    for maybe_shuttle_debug_folder in os.listdir(log_root_folder):
        if "shuttle-debug" not in maybe_shuttle_debug_folder:
            continue

        # 潜在可能的有效数据目录
        maybe_path = os.path.join(log_root_folder, maybe_shuttle_debug_folder)

        if not os.path.isdir(maybe_path):
            continue

        # pattern = re.compile(r".*[0-9]$")

        # 只读取log类型且格式为"shuttle-debug"的文件
        for single_log_file in os.listdir(maybe_path):
            if robot_id not in single_log_file:
                continue

            if is_file_in_time_range(os.path.join(maybe_path, single_log_file), time_range_in_mins, date_selected):
                yield os.path.abspath(os.path.join(maybe_path, single_log_file))


def get_file_in_time_range_pick(robot_id,  log_root_folder, time_range_in_mins=10, date_selected=datetime.now()):
    """ 遍历所有文件并保存所有日志文件

    Args:
        robot_id (string): 机器人编号
        log_root_folder (string): 日志文件根目录
        time_range_in_mins (int, optional): 基于选取日志的时间，确定选取日志范围的时间窗口
        date_selected (datetime, optional): 选取日志的时间

    Yields:
        generator(string): 目标文件路径的生成器
    """

    for root, dirs, files in os.walk(log_root_folder):
        for file_name in files:
            res = pick_load_file_match_res(file_name)
            log_file_full_path = os.path.abspath(os.path.join(root, file_name))

            if bool(res) and bool(re.search(rf"{robot_id}$", root)) and is_file_in_time_range(log_file_full_path, time_range_in_mins, date_selected):
                yield log_file_full_path


def get_file_in_time_range_for_download(log_root_folder, time_range_in_mins=10, date_selected=datetime.now(), **kwargs):
    """ 遍历所有文件并保存所有日志文件名,日志下载用

    Args:
        log_root_folder (string): 日志文件根目录
        time_range_in_mins (int, optional): 基于选取日志的时间，确定选取日志范围的时间窗口
        date_selected (datetime, optional): 选取日志的时间

    Yields:
        generator(string): 目标文件路径的生成器
    """
    if 'recursive' in kwargs and not kwargs['recursive']:
        for file in os.listdir(log_root_folder):
            log_file_full_path = os.path.abspath(os.path.join(log_root_folder, file))

            if os.path.isdir(log_file_full_path):
                continue

            if is_file_in_time_range(log_file_full_path, time_range_in_mins, date_selected):
                yield log_file_full_path

    if 'recursive' not in kwargs or ('recursive' in kwargs and kwargs['recursive']):
        for root, dirs, files in os.walk(log_root_folder):
            for file_name in files:
                log_file_full_path = os.path.abspath(os.path.join(root, file_name))

                if is_file_in_time_range(log_file_full_path, time_range_in_mins, date_selected):
                    yield log_file_full_path


def create_diagram(input_data, input_args):
    """生成图表

    Args:
        input_data (tuple): (timestamp_points(list),torque_values(list), cur_voltage_value(list))
        input_args (tuple): shell传入的参数
    """

    timestamp_points, cur_voltage_values, torque_values = input_data

    # 这步骤操作在 read_data_lines 的方法里直接做了,减少一次运算
    # log_datetime = np.vectorize(mdates.date2num)(timestamp_points)
    log_datetime = timestamp_points
    date_format = mdates.DateFormatter('%d-%m-%Y/%H:%M:%S')

    rect1 = [0.10, 0.5, 0.80, 0.45]

    # improve the resolution ratio
    fig = plt.figure(figsize=(20, 10), dpi=150)

    ax1 = plt.axes(rect1)

    ax1.xaxis_date()
    ax1.plot(log_datetime, torque_values)

    # set y tick major locator
    ax1.yaxis.set_major_locator(ticker.MultipleLocator(200))
    # ax1.yaxis.set_major_formatter(FormatStrFormatter('%0.1f'))

    # set y tick minor locator
    ax1.yaxis.set_minor_locator(ticker.MultipleLocator(10))
    # ax1.yaxis.set_minor_formatter(FormatStrFormatter('%0.1f'))

    # set y grid
    ax1.yaxis.grid(True, which='major', linestyle='--')

    # ax1.set_yticks(range(-1400, 1400, 200))
    ax1.xaxis.set_major_formatter(date_format)
    ax1.tick_params(axis='both', labelsize=7)

    ax1.set_ylabel("Torque", fontsize=10)
    plt.setp(ax1.get_xticklabels(), rotation=30, ha="right")


    # # 绘制电压时间子图
    rect2 = [0.10, 0.15, 0.80, 0.25]
    ax2 = plt.axes(rect2)

    ax2.xaxis_date()
    ax2.plot(log_datetime, cur_voltage_values, linewidth=1, c='red')

    # set y tick major locator
    ax2.yaxis.set_major_locator(ticker.MultipleLocator(5))

    # set y tick minor locator
    ax2.yaxis.set_minor_locator(ticker.MultipleLocator(1))

    # set y grid
    ax2.yaxis.grid(True, which='major', linestyle='--')
    ax2.yaxis.grid(True, which='minor', linestyle=':')

    ax2.xaxis.set_major_formatter(date_format)

    plt.setp(ax2.get_xticklabels(), rotation=30, ha="right")


    ax2.set_ylabel("Voltage (V)", fontsize=10)


    plt.savefig(input_args.output_file)


    if input_args.plot_show:
        plt.show()


def pick_load_file_match_res(file_name):
    """匹配pick的负载log类型的文件名

    Args:
        file_name (string): log文件名

    Returns:
        match objects: Match objects
    """
    log_name_pattern = re.compile( r"""
                                    ^load-                   # 以load开头
                                    \d{4}-\d{2}-\d{2}        # 日期
                                    \.log                    # load-2022-08-19.log
                                    (?P<log_serial_number>\.\d)?     # 如load-2022-08-18.log.1.gz
                                    (?P<log_zip_suffix>\.gz)?     # 如load-2022-08-18.log.1.gz
                                    """, re.X)

    return log_name_pattern.search(file_name)


def find_datetime_in_line(line):
    """匹配log文件一行中的时间戳

    Args:
        line (string): log文件中的一行

    Returns:
        string: 时间戳的字符串格式
    """

    timestamp_pattern = re.compile(r"(\d{4}-\d{2}-\d{2}T\d{2}:\d{2}:\d{2}\.\d{6})")
    m = re.search(timestamp_pattern, line)
    return m[0]


def preprocessing_output_directory(output_directory):
    """预处理输出目录

    Args:
        output_directory (string): 输出目录

    Returns:
        string: 输出目录
    """
    try:
        if not os.path.exists(output_directory):
            os.makedirs(output_directory)

        files = os.listdir(output_directory)
        # file name like: hostname-20210913175500.log.tar.gz
        # hostname may contain '-'
        # get time by regular, sort by time
        files.sort(key=lambda x: re.search(r"\d{14}", x).group())

        # delete all the oldest files if the number of files is larger than 10
        if len(files) >= 10:
            for file in files[:-9]:
                os.remove(os.path.join(output_directory, file))

        return output_directory



    except Exception as error:
        print(error)


def upload_to_obs(upload_file):
    # check if {User}/obsutil/push.sh exists
    try:
        # ask if want to upload to obs
        # print beautiful box around the tip in terminal

        # the box length should auto adjust to the length of the tip


        print("""
        \033[1;31m
        +-------------------------------------------------+
            是否要将打包的日志文件上传到云盘？ (y/n) .
        +-------------------------------------------------+
        \033[0m
        """.format(upload_file=upload_file))

        print(f"是否要将 {upload_file} 上传到云盘? (y/n)")
        if input() == 'y':
            obsutil_push_script_path = os.path.join(os.path.expanduser('~'), 'obsutil', 'push.sh')

            if not os.path.exists(obsutil_push_script_path):
                print(f"{obsutil_push_script_path} 不存在, 请联系管理员安装")
                return
            # run {User}/obsutil/push.sh upload_file
            os.system(f"{obsutil_push_script_path} {upload_file}")
        else:
            print(f"已选择跳过上传 {upload_file} 到云盘")

    except Exception as error:
        print(f"Error: {error}")



import unittest
import os
from unittest.mock import patch
import json
from src.v_t_chart import *
from datetime import datetime
from dateutil.parser import parse


class TestReadPickData(unittest.TestCase):

    def test_walk_folder(self):
        files = get_file_in_time_range_pick("474E50", os.path.relpath("./test/test_data/pick"), 1, parse("2022-09-03 14:23:59"))
        file_list = list(files)
        self.assertEqual(len(file_list), 1)
        self.assertIn(os.path.abspath("./test/test_data/pick/474E50/load-2022-09-03.log.gz"), file_list)


if __name__ == "main":
    unittest.main()



from setuptools import setup, find_packages
import os

try: # for pip >= 10
    from pip._internal.req import parse_requirements
except ImportError: # for pip <= 9.0.3
    from pip.req import parse_requirements

# get requirements from requirements.txt
install_reqs = parse_requirements("requirements.txt", session=False)
requirements = list(install_reqs)


try:
    requirements = [str(ir.req) for ir in requirements]
except:
    requirements = [str(ir.requirement) for ir in requirements]

# print(f"++++++++++++++++++++++++++++requirements: {requirements}")


setup(
    packages=find_packages(),
    install_requires = requirements,
    entry_points={
        'console_scripts': [
            'hc_log_tools = src.entry:entry',
        ]
    },
)


"""This is the description.

  example: src/pack_log.py --prd_type=sort --time_range=100  --timestamp="2022-09-13 17:55:00" --log_root_dir=/tmp/66log
"""
from src.log_utility import *
import argparse
import tarfile
import os
from datetime import datetime
from alive_progress import alive_bar



def init_input_args(sub_parser):
    """解析shell传入的参数, 返回结果
    Returns:
        parser Namespace: 返回解析后的结果
    """
    # sub_parser.add_argument("--prd_type", help="产品类型：sort or pick", required=False)
    sub_parser.add_argument(
        "--time_range",
        type=int,
        help="希望抓取日志的有效时间范围：默认前后30分钟。截取的日志范围，距离时间点（--timestamp）前后多少分钟。",
        default=30,
        choices=range(1, 60),
        metavar="[1 ~ 60]",
    )
    sub_parser.add_argument("--log_root_dir", help="log 文件的根目录", required=True)
    sub_parser.add_argument(
        "--timestamp",
        help="时间点, 例如：2022-09-05 09:53:10。 默认为当前时间",
        default=datetime.now().strftime("%m-%d-%Y %H:%M:%S"),
        required=True
    )

    sub_parser.add_argument(
        "--output_directory",
        help="输出文件的目录, 默认路径：/tmp/log_trans/",
        default="/tmp/log_trans/",
        required=False
    )

    # DON'T want to allow --feature and --no-feature at the same time
    feature_recursive_parser = sub_parser.add_mutually_exclusive_group(required=False)
    feature_recursive_parser.add_argument('--recursive', dest='recursive', action='store_true', help='递归处理log文件目录的子目录, 默认激活该选项')
    feature_recursive_parser.add_argument('--no-recursive', dest='recursive', action='store_false', help='不处理log文件目录的子目录')
    sub_parser.set_defaults(recursive=True)

    feature_compress_parser = sub_parser.add_mutually_exclusive_group(required=False)
    feature_compress_parser.add_argument('--compress', dest='compress', action='store_true', help="压缩输出文件, 默认激活该选项")
    feature_compress_parser.add_argument('--no-compress', dest='compress', action='store_false', help="不压缩输出文件")
    sub_parser.set_defaults(compress=True)

    return sub_parser


def init_input_args_warp(sub_parser):
    res_parser = init_input_args(sub_parser)
    return res_parser.parse_args()


def pack_log(input_args):
    """找到指定的log，并打包
    Args:
        input_args: shell 传入的参数
    """

    preprocessing_output_directory(input_args.output_directory)

    print(f"input_args.recursive: {input_args.recursive}")
    log_files = set(
        get_file_in_time_range(
            input_args, download_log=True, recursive=input_args.recursive
        )
    )
    print(f"log_files: {log_files}")

    if len(log_files) == 0:
        print("no log files in range")
        return

    print(f"totlal {len(log_files)} files, compress start ...")

    # get current datetime like 20210913175500
    cur_datetime = datetime.now().strftime("%Y%m%d%H%M%S")
    # get system hostname
    hostname = os.popen("hostname").read().strip()
    output_file_name = f"{hostname}-{cur_datetime}.log.tar.gz"
    # join args.output_directory and OUTPUT_FILE_NAME
    output_file_path = os.path.join(input_args.output_directory, output_file_name)


    # if input_args.compress, tar with gzip
    if input_args.compress:
        with alive_bar(len(log_files),  title='pack with compress') as bar:
            with tarfile.open(output_file_path, "w:gz") as tar:
                for log_file in log_files:
                    try:
                        tar.add(log_file)
                        bar()
                    except FileNotFoundError:
                        print(f"file {log_file} is not exist")

    # else, tar without gzip
    else:
        with alive_bar(len(log_files),  title='pack without compress') as bar:
            with tarfile.open(output_file_path, "w") as tar:
                for log_file in log_files:
                    try:
                        tar.add(log_file)
                        bar()
                    except FileNotFoundError:
                        print(f"file {log_file} is not exist")

    # terminal process bar for compress with alive-progress
    # show the output file size with MB, precision=2
    print(f"compress finished, output file: {output_file_path}, size: {os.path.getsize(output_file_path) / 1024 / 1024:.2f} MB")

    print(f"compress finished, total {len(log_files)} files output file: {output_file_path}")
    print(f"package size: {os.path.getsize(output_file_path) / 1024 / 1024:.2f} MB")

    upload_to_obs(output_file_path)

if __name__ == "__main__":
    parser = argparse.ArgumentParser(
        description=__doc__, formatter_class=argparse.RawDescriptionHelpFormatter
    )
    input_args = init_input_args_warp(parser)
    pack_log(input_args)



import unittest
from unittest import mock
from unittest.mock import patch
from src.log_utility import *
from datetime import datetime
import os


class TestDownload(unittest.TestCase):

    def test_get_file_in_time_range(self):
        # python3 src/pack_log.py --prd_type=sort --time_range=1  --timestamp="2022-09-13 20:05:00" --log_root_dir /home/deployer/Projects/hc_sort/current/logs
        mocked_input_args = mock.MagicMock()
        mocked_input_args.prd_type = "sort"
        mocked_input_args.log_root_dir =  "./test/test_data/pick/all"
        mocked_input_args.time_range = 1
        mocked_input_args.timestamp = "2022-09-14 17:20:00"

        res = set(get_file_in_time_range(mocked_input_args, download_log=True))
        self.assertIn(os.path.abspath('./test/test_data/pick/all/all-logs-2022-09-14-17.log'), res)

    def test_file_should_not_in_range(self):
        # python3 src/pack_log.py --prd_type=sort --time_range=1  --timestamp="2022-09-13 20:05:00" --log_root_dir /home/deployer/Projects/hc_sort/current/logs
        mocked_input_args = mock.MagicMock()
        mocked_input_args.prd_type = "sort"
        mocked_input_args.log_root_dir =  "./test/test_data/pick/all"
        mocked_input_args.time_range = 1
        mocked_input_args.timestamp = "2022-09-14 16:20:00"

        res = set(get_file_in_time_range(mocked_input_args, download_log=True))
        self.assertNotIn(os.path.abspath('./test/test_data/pick/all/all-logs-2022-09-14-17.log'), res)

        mocked_input_args.timestamp = "2022-09-14 18:20:00"
        res = set(get_file_in_time_range(mocked_input_args, download_log=True))
        self.assertNotIn(os.path.abspath('./test/test_data/pick/all/all-logs-2022-09-14-17.log'), res)

    def test_log_duriation_file_in_range(self):
        # python3 src/pack_log.py --prd_type=sort --time_range=1  --timestamp="2022-09-13 20:05:00" --log_root_dir /home/deployer/Projects/hc_sort/current/logs
        mocked_input_args = mock.MagicMock()
        mocked_input_args.prd_type = "sort"
        mocked_input_args.log_root_dir =  "./test/test_data/pick/all"
        mocked_input_args.time_range = 1
        mocked_input_args.timestamp = "2022-09-13 15:20:00"
        mocked_input_args.recursive = True

        res = set(get_file_in_time_range(mocked_input_args, download_log=True, recursive=mocked_input_args.recursive))
        self.assertIn(os.path.abspath('./test/test_data/pick/all/all-logs-2022-09-13-18.log'), res)
        self.assertIn(os.path.abspath('./test/test_data/pick/all/sub_dir/all-logs-2022-09-13-18_sub.log'), res)

    def test_log_duriation_file_in_range_not_recursive(self):
        # python3 src/pack_log.py --prd_type=sort --time_range=1  --timestamp="2022-09-13 20:05:00" --log_root_dir /home/deployer/Projects/hc_sort/current/logs
        mocked_input_args = mock.MagicMock()
        mocked_input_args.prd_type = "sort"
        mocked_input_args.log_root_dir =  "./test/test_data/pick/all"
        mocked_input_args.time_range = 1
        mocked_input_args.timestamp = "2022-09-13 15:20:00"
        mocked_input_args.recursive = False

        res = set(get_file_in_time_range(mocked_input_args, download_log=True, recursive=mocked_input_args.recursive))
        self.assertIn(os.path.abspath('./test/test_data/pick/all/all-logs-2022-09-13-18.log'), res)
        self.assertNotIn(os.path.abspath('./test/test_data/pick/all/sub_dir/all-logs-2022-09-13-18_sub.log'), res)


# run: python3 -m test.test_can_io_config.sort_excel_test
if __name__ == '__main__':
    unittest.main()



import os
from datetime import datetime, timedelta
import json
import warnings
import gzip
from dateutil.parser import parser
import argparse
from src.log_utility import *

# import numpy as np

# 屏蔽所有 告警
warnings.filterwarnings("ignore")

# 默认图片存储地址，如不存在则新建该地址
OUTPUT_FILE = r"/tmp/chart/voltage_torque.jpg"

if not os.path.exists("/tmp/chart/"):
    os.mkdir("/tmp/chart/")


# 参数分别为机器人ID，时间戳，log根目录，输出图片保存路径
def generate_chart(input_args):
    """生成图表总入口

    Args:
        args: 通过命令行传入的参数
    """

    data_files = set(get_file_in_time_range(input_args))

    if not data_files:
        raise RuntimeError(f"没有时间点 {input_args.timestamp} 的相关数据")

    diagram_input_data = list(
        collection_data_for_plot(
            data_files, parse(input_args.timestamp), input_args.time_range
        )
    )

    if len(data_files) == 0:
        raise RuntimeError("所选时间范围内没有数据")

    # 绘图
    create_diagram(diagram_input_data, input_args)

    print(f"图表已生成，保存路径：{input_args.output_file}")


def collection_data_for_plot(data_files, date_selected, time_range):
    """根据要求的时间收集数据

    Args:
        data_files (set): 目标log的集合, 单次查询可能获取跨文件的数据
        date_selected (datetime): 要分析的日志的时间戳
        time_range (int): 分钟，截取的日志范围，距离当前时间前后多少分钟
    """
    res = (list(), list(), list())

    for file in data_files:
        if file.endswith("gz"):
            with gzip.open(file, mode="rt") as data_f:
                one_file_res = read_data_lines(data_f, date_selected, time_range)
                res = (
                    res[0] + one_file_res[0],
                    res[1] + one_file_res[1],
                    res[2] + one_file_res[2],
                )
        else:
            with open(file, mode="r") as data_f:
                one_file_res = read_data_lines(data_f, date_selected, time_range)
                res = (
                    res[0] + one_file_res[0],
                    res[1] + one_file_res[1],
                    res[2] + one_file_res[2],
                )

    return res


def init_input_args(sub_parser):
    sub_parser.add_argument("--prd_type", help="产品类型：sort or pick", required=True)
    sub_parser.add_argument(
        "--time_range",
        type=int,
        help="产品类型：分钟，截取的日志范围，距离时间点（--timestamp）前后多少分钟。默认1分钟",
        choices=range(1, 6),
        metavar="[1 ~ 6]",
        required=True,
    )
    sub_parser.add_argument("--log_root_dir", help="log 文件的根目录", required=True)
    sub_parser.add_argument(
        "--timestamp",
        help="时间点, 例如：2022-09-05 09:53:10。 默认为当前时间",
        default=datetime.now().strftime("%m-%d-%Y %H:%M:%S"),
    )
    sub_parser.add_argument("--robot_id", help="目标机器人的编号，例如：F48A57S", required=True)
    sub_parser.add_argument(
        "--output_file", help=f"输出文件的路径, 默认路径：{OUTPUT_FILE}", default=OUTPUT_FILE
    )
    sub_parser.add_argument("--plot_show", action="store_true", help="是否直接输出图像, 仅供调试使用")
    return sub_parser


def init_input_args_warp(sub_parser):
    res_parser = init_input_args(sub_parser)
    return res_parser.parse_args()


if __name__ == "__main__":
    res_json = {"code": 0, "message": "操作成功"}

    try:
        parser = argparse.ArgumentParser()
        args = init_input_args_warp(parser)
        generate_chart(args)

    except Exception as e:
        res_json["code"] = 1
        res_json["message"] = str(e)

    print(json.dumps(res_json, ensure_ascii=False, indent=4, sort_keys=True))

    # args = init_input_args()
    # generate_chart(args)
